#include <stdio.h>
#include <string.h>
#include "rs232.h"
#include "vna.h"
#include "bits.h"
#include "uart.h"
#include <unistd.h>
#include <stdint.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <strings.h>
#include <errno.h>




void
main_cpu_reset ( void )
{
  uint8_t buf[5];
  
  /* reset CPU */
  buf[0] = VNA_MSG_CPU_RESET;
  buf[1] = 0x5A;
  buf[2] = 0x69;
  buf[3] = 0xA5;

  /* send to vna-232 */
  rs232_tx(buf,4);

  printf("info: sent reset command\n");
}



/*
** Transmit Power Takeoff Information msg (PGN 65264)
*/
void
main_j1939_tx ( void )
{
  uint8_t cnt;
  uint8_t buf[20];
  
  buf[0] = VNA_MSG_TX_J1939;
  buf[1] = 0;      // port
  buf[2] = 0;      // pgn 23-16
  buf[3] = 0xFE;   // pgn 15 - 8
  buf[4] = 0xF0;   // pgn 7 - 0
  buf[5] = 10;     // dst
  buf[6] = 11;     // src
  buf[7] = 3;      // pri

  /* set to 'not availble' for since this is a test msg */
  for( cnt = 0; cnt < 8; cnt++ )
    buf[8+cnt] = 0xFF;

  rs232_tx(buf,16);
  printf("info: transmitted power takeoff information onto j1939. (pgn=65264)\n");
}



/*
** Transmit Mass Air Flow (PID 132)
*/
void
main_j1587_tx ( void )
{
  uint8_t buf[20];
  
  buf[0] = VNA_MSG_TX_J1587;
  buf[1] = 145;   // mid
  buf[2] = 0;     // pid 15 - 8 
  buf[3] = 132;   // pid 7 - 0
  buf[4] = 3;     // pri
  buf[5] = 40;    // data 0
  buf[6] = 50;    // data 1

  rs232_tx(buf,7);
  printf("info: transmitted mass air flow information onto j1587. (pid=132)\n");
}



/*
** Add filter for PID 133 (Average fuel rage)
*/
void
main_filter_add_133 ( void )
{
  uint8_t buf[5];
  
  /* add filter for cruise control/vehicle speed PGN */
  buf[0] = VNA_MSG_FA_J1587;
  buf[1] = 0;     // pid 15 - 8
  buf[2] = 133;   // pid 7 - 0

  /* send to vna-232 */
  rs232_tx(buf,3);

  printf("info: added average fuel rate j1587 filter(pid=133)\n");
}



/*
** Add filter for PGN 65265 (Cruise Control/Vehicle Speed)
*/
void
main_filter_add_65265 ( void )
{
  uint8_t buf[5];
  
  /* add filter for cruise control/vehicle speed PGN */
  buf[0] = VNA_MSG_FA_J1939;
  buf[1] = 0;      // port 0
  buf[2] = 0;      // pgn 23-16
  buf[3] = 0xFE;   // pgn 15 - 8
  buf[4] = 0xF1;   // pgn 7 - 0

  /* send to vna-232 */
  rs232_tx(buf,5);

  printf("info: added cruise control/vehicle speed j1939 filter(pgn=65265)\n");
}



int main(int argc, char* argv[])
{
  uint8_t val;

  /* quick check for correct arguments */
  if(argc<2) {
    fprintf(stderr, "  usage: pc232 <path to uart>\n\n");
    fprintf(stderr, "example: pc232 /dev/ttyUSB0\n\n");
    exit(-1);
  }

  /* open port */
  fprintf(stderr, "opening %s\n", argv[1]);
  if( uart_init(argv[1]) )
    exit(-1);

  /* add our filters */
  main_filter_add_65265();
  main_filter_add_133();

  /* transmit sample msgs */
  main_j1939_tx();
  main_j1587_tx();

  while( 1 ) {

    /* read single byte from port and unstuff */
    if( uart_rx(&val) == 0 )
      rs232_update(val);

    usleep(1);
  }

  return 0;
}
